//
//  HUDNode.m
//  SuperKoalio
//
//  Created by Jake Gundersen on 3/1/14.
//  Copyright (c) 2014 Razeware. All rights reserved.
//

#import "HUDNode.h"
#import "Joystick.h"

@interface HUDNode()

@property (nonatomic, strong) Joystick *joystick;

@property (nonatomic, strong) SKSpriteNode *aButton;
@property (nonatomic, strong) SKSpriteNode *bButton;

@property (nonatomic, strong) UITouch *dashTouch;

@end

@implementation HUDNode

- (instancetype)initWithSize:(CGSize)size {
    if ((self = [super init])) {
        //2
        self.userInteractionEnabled = YES;
        //3
        SKSpriteNode *joyBack = [SKSpriteNode spriteNodeWithImageNamed:@"dpad"];
        SKSpriteNode *joyFront = [SKSpriteNode spriteNodeWithImageNamed:@"joystick"];
        joyBack.alpha = 0.5;
        joyFront.alpha = 0.7;
        //4
        self.joystick = [Joystick joystickWithThumb:joyFront andBackdrop:joyBack];
        //5
        self.joystick.position = CGPointMake(90, 160);
        [self addChild:self.joystick];
        //6
        self.aButton = [SKSpriteNode spriteNodeWithImageNamed:@"buttonA"];
        self.aButton.position = CGPointMake(size.width - 40, 120);
        [self addChild:self.aButton];
        
        self.bButton = [SKSpriteNode spriteNodeWithImageNamed:@"buttonB"];
        self.bButton.position = CGPointMake(size.width - 140, 120);
        [self addChild:self.bButton];
        
        //Add hardware controller code here
        
        //Add observers here
        
    }
    return self;
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    for (UITouch *touch in touches) {
        //1
        CGPoint touchLocation = [touch locationInNode:self];
        //2
        if (CGRectContainsPoint(self.bButton.frame, touchLocation)) {
            self.shouldDash = YES;
            self.dashTouch = touch;
            self.bButton.alpha = 0.7;
        }
        //3
        if (CGRectContainsPoint(self.aButton.frame, touchLocation)) {
            self.aPushed = YES;
            self.aButton.alpha = 0.7;
        }
    }
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    for (UITouch *touch in touches) {
        //1
        CGPoint touchLocation = [touch locationInNode:self];
        CGPoint previousTouchLocation = [touch previousLocationInNode:self];
        //2
        if (CGRectContainsPoint(self.aButton.frame, previousTouchLocation) &&
            !CGRectContainsPoint(self.aButton.frame, touchLocation)) {
            self.aPushed = NO;
            self.aButton.alpha = 1.0;
        }
        //3
        if (!CGRectContainsPoint(self.aButton.frame, previousTouchLocation) &&
            CGRectContainsPoint(self.aButton.frame, touchLocation)) {
            self.aPushed = YES;
            self.aButton.alpha = 0.7;
        }
    }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    for (UITouch *touch in touches) {
        //1
        if (self.dashTouch == touch) {
            self.shouldDash = NO;
            self.dashTouch = nil;
            self.bButton.alpha = 1.0;
        }
        //2
        CGPoint touchLocation = [touch locationInNode:self];
        if (CGRectContainsPoint(self.aButton.frame, touchLocation)) {
            self.aPushed = NO;
            self.aButton.alpha = 1.0;
        }
    }
}

- (float)xJoystickVelocity {
    return self.joystick.velocity.x / 60.0;
}

@end
